@@ -3,19 +3,15 @@ import csv
 import logging
 import os
 import re
-import signal
 import sys
-from contextlib import contextmanager
 from pathlib import Path
 from typing import Optional, NoReturn, Dict, List, Any
 
-from heuristics import lm_cut
-from pyperplan import search_plan
-from search import greedy_best_first_search
 from task import Operator
 
 from sam_learner.core import TrajectoryGenerator
 from sam_learner.sam_models import Trajectory
+from .fast_downward_solver import FastDownwardSolver
 
 PLAN_SEARCH_TIME_LIMIT = int(os.environ.get('SEARCH_TIME', "300"))
 VALID_STATUS = "valid plan!"
@@ -40,22 +36,6 @@ class ValidationResult:
 		self.failed_action_name = failed_action_name
 
 
-class TimeoutException(Exception): pass
-
-
-@contextmanager
-def time_limit(seconds):
-	def signal_handler(signum, frame):
-		raise TimeoutException("Timed out!")
-
-	signal.signal(signal.SIGALRM, signal_handler)
-	signal.alarm(seconds)
-	try:
-		yield
-	finally:
-		signal.alarm(0)
-
-
 def extract_failed_action_name(error_message) -> str:
 	"""Extract the failed action name from the error message.
 
@@ -84,10 +64,12 @@ class DomainValidator:
 
 	logger: logging.Logger
 	expected_domain_path: str
+	fast_downward_solver: FastDownwardSolver
 
 	def __init__(self, expected_domain_path: str):
 		self.logger = logging.getLogger(__name__)
 		self.expected_domain_path = expected_domain_path
+		self.fast_downward_solver = FastDownwardSolver()
 
 	def write_plan_file(self, plan_actions: List[Operator], solution_path: Path) -> NoReturn:
 		"""Write the plan that was created using the learned domain file.
@@ -109,42 +91,22 @@ class DomainValidator:
 		"""
 		successful_plans = {}
 		self.logger.info("Solving the test set problems using the learned domain!")
+		fast_downward_execution_script_path = Path(test_set_directory) / "solver_execution_script.sh"
+		self.fast_downward_solver.write_batch_and_execute_solver(
+			output_file_path=fast_downward_execution_script_path,
+			test_set_directory_path=Path(test_set_directory),
+			problems_regex="*.pddl",
+			domain_file_path=tested_domain_file_path
+		)
 		for problem_file_path in Path(test_set_directory).glob("*.pddl"):
-			self.logger.debug(f"Solving problem - {problem_file_path.stem}")
-			try:
-				with time_limit(PLAN_SEARCH_TIME_LIMIT):
-					self.search_plan_with_time_limit(
-						problem_file_path, successful_plans, test_set_directory,
-						tested_domain_file_path)
-
-			except TimeoutException as e:
-				self.logger.info("Solver execution timed out!")
-				successful_plans[str(problem_file_path)] = TIMEOUT_WHILE_SOLVING_ERROR
+			solution_file_path = problem_file_path.parent / f"{problem_file_path.stem}_plan.solution"
+			if solution_file_path.exists():
+				successful_plans[str(problem_file_path)] = str(solution_file_path)
+			else:
+				successful_plans[str(problem_file_path)] = PROBLEM_NOT_SOLVABLE_ERROR
 
 		return successful_plans
 
-	def search_plan_with_time_limit(
-			self, problem_file_path: Path, successful_plans: Dict[str, Optional[str]],
-			test_set_directory: str, tested_domain_file_path: str) -> NoReturn:
-		"""Search for a plan for the specific problem with a time limit so that the process won't go on forever.
-
-		:param problem_file_path: the path to the problem file.
-		:param successful_plans: the dictionary containing the successful plans.
-		:param test_set_directory: the directory containing the problems to solve.
-		:param tested_domain_file_path: the domain used to solve the problems.
-		"""
-		plan_actions = search_plan(tested_domain_file_path, problem_file_path, greedy_best_first_search,
-								   lm_cut.LmCutHeuristic)
-		if plan_actions is not None and len(plan_actions) > 0:
-			self.logger.info(f"Solution Found from problem -{problem_file_path.stem}!")
-			solution_path = Path(test_set_directory, f"{problem_file_path.stem}_plan.solution")
-			self.write_plan_file(plan_actions, solution_path)
-			successful_plans[str(problem_file_path)] = str(solution_path)
-
-		else:
-			self.logger.info(f"Could not find a solution for problem - {problem_file_path.stem}")
-			successful_plans[str(problem_file_path)] = PROBLEM_NOT_SOLVABLE_ERROR
-
 	def validate_plan(self, problem_path: str, plan_path: str) -> ValidationResult:
 		"""Validate the correctness of the learned domain against the domain learned using the learner algorithm.
 
@@ -189,15 +151,19 @@ class DomainValidator:
 				writer.writerow(data)
 
 	@staticmethod
-	def clear_plans(created_plans: List[str]) -> NoReturn:
+	def clear_plans(created_plans: List[str], test_set_directory_path: str) -> NoReturn:
 		"""Clears the directory from the plan files so that the next iterations could continue.
 
 		:param created_plans: the paths to the plans that were created in this iteration.
+		:param test_set_directory_path: the path to the test set directory containing the solver execution log.
 		"""
 		for plan_path in created_plans:
 			if plan_path is not None:
 				os.remove(plan_path)
 
+		for solver_output_path in Path(test_set_directory_path).glob("*.out"):
+			os.remove(solver_output_path)
+
 	def on_validation_success(
 			self, output_statistics_path: str,
 			test_set_statistics: List[Dict[str, Any]]) -> NoReturn:
@@ -238,6 +204,7 @@ class DomainValidator:
 				if domain_successful_plans[problem_path] in POTENTIAL_PLANNING_ERRORS:
 					domain_statistics["plan_generated"] = False
 					domain_statistics["plan_validation_status"] = domain_successful_plans[problem_path]
+					validated_plans.append(False)
 					test_set_statistics.append(domain_statistics)
 					continue
 
@@ -248,7 +215,7 @@ class DomainValidator:
 				validated_plans.append(validation_status.status == VALID_STATUS)
 				test_set_statistics.append(domain_statistics)
 
-			self.clear_plans(plan_paths)
+			self.clear_plans(plan_paths, test_set_directory_path)
 			if all(validated_plans):
 				self.on_validation_success(output_statistics_path, test_set_statistics)
 				return
